package com.ruoyi.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.ruoyi.app.controller.base.AppBaseController;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.domain.TbFamilyMember;
import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbFamilyMemberService;
import com.ruoyi.app.service.ITbFamilyService;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 家族成员管理
 *
 * @author Alex
 * @date 2020-09-24
 */
@Api(value = "家族成员管理",tags = "家族成员管理")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/member" )
public class FamilyMemberController extends AppBaseController {

    private final ITbFamilyMemberService memberService;
    private final ITbAppUserService appUserService;
    private final ITbFamilyService familyService;
    private final ITbMemberMiddleService middleService;

    @ApiOperation("我的家族成员列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "当前页",paramType="int"),
            @ApiImplicitParam(name = "pageSize", value = "每页行数",paramType="int")
    })
    @GetMapping("/list")
    public TableDataInfo list(Integer pageNum, Integer pageSize) {
        // 登录用户
        TbAppUser user = getLoginUser().getUser();
        // 获取成员信息
        AppMemberVo memberVo = appUserService.getAppMember(user.getId().toString(),"");
        if (memberVo == null) {
            return new TableDataInfo();
        }
        // 获取默认家族信息
        TbFamily family = familyService.myFamily(memberVo.getMemberId());
        // 获取家族关联成员的信息
        List<TbMemberMiddle> middles = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
            .eq(TbMemberMiddle::getFamilyId, family.getId())
        );
        if (middles.size() <= 0) {
            return new TableDataInfo();
        }
        List<Long> ids = new ArrayList<>();
        middles.forEach(item -> {
            ids.add(item.getMemberId());
        });
        // 根据家族获取成员信息
        pageNum = pageNum == null ? 1 : pageNum;
        pageSize = pageSize == null ? 10 : pageSize;
        PageHelper.startPage(pageNum, pageSize, "py_first asc");
        List<TbFamilyMember> list = memberService.list(new LambdaQueryWrapper<TbFamilyMember>()
            .in(TbFamilyMember::getId, ids)
        );
        return getDataTable(list);
    }


    //家族树

}
